import sys, os
import pygame as pg
import socket as sock
import json

pg.init()
screen = pg.display.set_mode((600, 350), flags=pg.RESIZABLE)
pg.display.set_caption("RudlEngine: Debug Mode")

BLACK = (0, 0, 0)
HOST_DEBUG = "127.0.0.1"
PORT_DEBUG = 5050


font = pg.font.Font(None, 36)

server = sock.socket(sock.AF_INET, sock.SOCK_STREAM)
server.bind((HOST_DEBUG, PORT_DEBUG))
server.listen(1)

server.settimeout(2.5)

try:
    conn, addr = server.accept()
except sock.timeout:
    pg.display.message_box("Sorry! But you need to create config")
    sys.exit(1)


run = True
clock = pg.time.Clock()
buffer = ""
counter = 0

while run:
    screen.fill(BLACK)
    clock.tick(60)

    try:
        data = conn.recv(1024)
        if not data:
            break

        buffer += data.decode("utf-8")

        while "\n" in buffer:
            json_str, buffer = buffer.split("\n", 1)
            if not json_str.strip():
                continue
            try:
                json_data = json.loads(json_str)
                x = screen.get_width() * 0.2
                y = 20
                offset_y = 25

                for i, (key, value) in enumerate(json_data.items()):
                    text_surface = font.render(f"{key}: {value}", True, (255, 255, 255))
                    screen.blit(text_surface, (x, y + i * offset_y))
            except json.JSONDecodeError as e:
                pass

    except ConnectionResetError:
        conn.close()
        conn, addr = server.accept()
        buffer = ""

    for event in pg.event.get():
        if event.type == pg.QUIT or (event.type == pg.KEYDOWN and event.key == pg.K_ESCAPE):
            run = False

        

    pg.display.flip()

pg.quit()
conn.close()
server.close()
sys.exit(0)



import glm
import pygame as pg


class CameraBase:
    def __init__(self, app):
        self.app = app
        self.offset = glm.vec2(0, 0)

        self.vision = None

    def scroll(**kwargs):
        pass

    def debug_render(self, **kwargs):
        pass



class RegionCamera:
    def __init__(self, app):
        self.app = app
        self.region_rect = pg.FRect(0, 200, 100, 600)

        self.offset = glm.vec2(0.0, 0.0)
        self.vision = pg.FRect(0, 0, 0, 0)

    def scroll(self, target_rect, surface, **kwargs):
        surface = surface if surface else self.app.screen

        min_x = kwargs.get("min_x", 0)
        min_y = kwargs.get("min_y", 0)

        max_x = kwargs.get("max_x", 2000)
        max_y = kwargs.get("max_y", 2000)

        velocity = kwargs.get("velocity", glm.vec2(150, 0))


        if not self.region_rect.contains(target_rect):
            if target_rect.left < self.region_rect.left:
                self.region_rect.x -= velocity.x * self.app.delta_time
            elif target_rect.right > self.region_rect.right:
                self.region_rect.x += velocity.x * self.app.delta_time

            if target_rect.top < self.region_rect.top:
                self.region_rect.y -= velocity.y * self.app.delta_time
            elif target_rect.bottom > self.region_rect.bottom:
                self.region_rect.y += velocity.y * self.app.delta_time



        self.vision.x = self.offset.x - 400
        self.vision.y = self.offset.y - 250
        self.vision.w = surface.get_width()
        self.vision.h = surface.get_height()

     
        self.offset.x = max(min_x, min(self.region_rect.x - surface.get_width() // 2 + 500, max_x))
        self.offset.y = max(min_y, min(self.region_rect.y - surface.get_height() // 2 + 200, max_y))







import pygame as pg
from rudleg.textures.texture import give_surfture


class DecursiveAnimation2D:
    pass


class RecursiveAnimation2D:
    def __init__(self, frames: list, ctx, repeat=True, fps=10):
        self.frames_texture = [give_surfture(frame, ctx=ctx) for frame in frames]
        self.index = 0

    
        self.repeat = repeat
        self.duration = 1000 / fps
        self.last_time = pg.time.get_ticks()
        


    def update(self, paused=False):
        now = pg.time.get_ticks()

        if now - self.last_time >= self.duration:
            if not paused:
                self.index += 1

            if len(self.frames_texture) <= self.index:
                if self.repeat:
                    self.index = 0
                else:
                    self.index = len(self.frames_texture) - 1


            self.last_time = now

        
    def get_texture(self):
        return self.frames_texture[self.index]
    



import pygame as pg
import moderngl as mgl
import glm
import os


#from path str to texture
def give_texture(path, ctx, filter="near"):
    if os.path.exists(path):
        surface = pg.transform.flip(pg.image.load(path).convert_alpha(), flip_x=False, flip_y=True)
        surface_data = pg.image.tobytes(surface, "RGBA")

        texture = ctx.texture(surface.get_size(), 4, surface_data)
        
        if filter == "near":
            texture.filter = (mgl.NEAREST, mgl.NEAREST)
        elif filter == "line":
            texture.filter = (mgl.LINEAR, mgl.LINEAR)

        return texture
    else:
        raise FileNotFoundError(f"Undefined path - {path}")
    

#from surface to texture

def give_surfture(surface: pg.Surface, ctx, filter="near"):
    surface = pg.transform.flip(surface, flip_y=True, flip_x=False)
    surface_data = pg.image.tobytes(surface, "RGBA")

    texture = ctx.texture(surface.get_size(), 4, surface_data)
    if filter == "near":
            texture.filter = (mgl.NEAREST, mgl.NEAREST)
    elif filter == "line":
            texture.filter = (mgl.LINEAR, mgl.LINEAR)

    return texture




def give_program(vertex_path, fragment_path, ctx):
    vertex_shader = read_shader_file(path=vertex_path)
    fragment_shader = read_shader_file(path=fragment_path)

    return ctx.program(vertex_shader=vertex_shader, fragment_shader=fragment_shader)



def from_ndc_to_sdl(width, height) -> glm.mat4:
     return glm.mat4(
        glm.vec4(2.0/width, 0.0, 0.0, 0.0),
        glm.vec4(0.0, -2.0/height, 0.0, 0.0),
        glm.vec4(0.0, 0.0, 1.0, 0.0),
        glm.vec4(-1.0, 1.0, 0.0, 1.0)
    )
     




def subthat(which_pos: tuple, which_size: tuple, original_surface: pg.Surface):
    sub_surface = original_surface.subsurface(*which_pos, *which_size).copy()

    return sub_surface.convert_alpha()




#very usefull function
def read_shader_file(path):
    if os.path.exists(path):
        with open(path, "r", encoding="UTF-8") as f:
            return f.read()
        
    else:
        raise FileNotFoundError(f"Undefined path - {path}")


import pygame as pg
import moderngl as mgl
import glm
from abc import ABC, abstractmethod
from typing import Protocol, Dict, Any


from rudleg.johnson import Joshua


class AppAnotation(Protocol):
    ctx: mgl.Context
    
    data: Joshua
    data_read: Dict[Any, Any]

    verticles_2d: glm.array
    indices: glm.array

    delta_time: float
    clock: pg.Clock

    @property
    def width(self) -> int: ...
    
    @property
    def height(self) -> int: ...

    @property
    def get_scene(self) -> str: ...

    def switch_scene(self, scene_name_anotation: str) -> None: ...
    def close_game(self) -> None: ...
    def change_data(self, key_d, value_d) -> None: ...
    def debug_sendall(self, dict_debug) -> None: ...
    def take_screenshot(self, path_name) -> None: ...

    



class MainObject(ABC):

    @abstractmethod
    def update(**kwargs):
        pass

    @abstractmethod
    def handler_event(**kwargs):
        pass


    @abstractmethod
    def render(**kwargs):
        pass



class Converter2D(ABC):
    @abstractmethod
    def build_map(self):
        pass

    @abstractmethod
    def get_objects(self):
        pass






class Kinematic2D(ABC):
    @abstractmethod
    def __init__(self, x, y, w, h, app):
        self.app = app

        self.rect = pg.FRect(x, y, w, h)
        self.velocity = glm.vec2(0.0, 0.0)


    def init_on(self):
        pass

    def set_obj(self, **kwargs):
        pass

    def update(self, **kwargs):
        pass

    
    def handler_event(self, event):
        pass

    
    def collision(self, **kwargs):
        pass


    def collision_x(self, **kwargs):
        pass


    def collision_y(self, **kwargs):
        pass


    def __repr__(self):
        return "That Kinematic2D object"




class Static2D(ABC):
    @abstractmethod
    def __init__(self, x, y, w, h, app):
        self.app = app

        self.rect = pg.FRect(x, y, w, h)

    def init_on(self):
        pass


    def set_obj(self, **kwargs):
        pass



    def update(self, **kwargs):
        pass

    
    def handler_event(self, event):
        pass


    def collision(self, **kwargs):
        pass


    def collision_x(self, **kwargs):
        pass


    def collision_y(self, **kwargs):
        pass


    def __repr__(self):
        return "That Static2D object"







"""We use that class for TypeError"""
class RelativeTypeError(Exception):
    def __init__(self, message="You possible give wrong arguments type not a Rectangle"):
        super().__init__(message=message)

    

"""That class use for prohibition action of client who use that Engine with bad target"""
class ProhibitionError(Exception):
    def __init__(self, message="You use imports [sys, os] in your config with bad target. Please without negative"):
        super().__init__(message)


"""We use that class for bonus"""
class AdminStateError(Exception):
    def __init__(self, *args):
        super().__init__(*args)



class OperationError(Exception):
    def __init__(self, *args):
        super().__init__(*args)


class SmallResolutionError(Exception):
    def __init__(self, *args):
        super().__init__(*args)


"""That Exception was created for reason to kill program if attribute created before other main attribute and etc"""
class BeforeCreatedError(Exception):
    def __init__(self, *args):
        super().__init__(*args)









class RangeType:
    pass



class InputType:
    pass



class GolType:
    pass



class BackgroundType:
    pass





class FadeType:
    pass






class FontType:
    pass



class SwitchBoardType:
    pass


import json


class Joshua:
    def __init__(self, path):
        self.path = path

    def read_data(self):
        with open(self.path, "r", encoding="UTF-8") as f:
            return json.load(f)
        
    def save_data(self, new_data):
        with open(self.path, "w", encoding="UTF-8") as f:
            json.dump(new_data, f, indent=2)




class Jackson:
    def __init__(self, path):
        self.path = path

    def read(self):
        with open(self.path, "r", encoding="UTF-8") as f:
            return f.read()
        
    def write(self, text):
        with open(self.path, "a") as f:
            f.write(text)


import pygame as pg


class ButtonSimple:
    def __init__(self, x, y, w, h, app, img:pg.Surface|None=None, text="Write you text"):
        self.rect = pg.Rect(x, y, w, h)
        self.app = app

        self.img = img

        self.hovered = False
        self.clicked = False
        self.timer = 0


    def handler_event(self, event):
        pass


    def update(self, **kwargs):
        mouse_pos = pg.mouse.get_pos()

       
        self.hovered = self.rect.collidepoint((int(mouse_pos[0]), int(mouse_pos[1])))





class ButtonState(ButtonSimple):
    def __init__(self, x, y, w, h, app, game_state, img=None, text="Write you text"):
        super().__init__(x, y, w, h, app, img, text)

        self.game_state = game_state

    def handler_event(self, event):
        if event.type == pg.MOUSEBUTTONDOWN:
            if event.button == 1 and self.hovered:
                self.clicked = True
    
    def update(self, timer_gone, func_fade=None):
        mouse_pos = pg.mouse.get_pos()

        self.hovered = self.rect.collidepoint((int(mouse_pos[0]), int(mouse_pos[1])))

        if self.clicked:
            self.timer += self.app.delta_time
            if func_fade is not None: func_fade()

        if self.timer >= timer_gone:
            self.app.switch_scene(self.game_state)
    

    


class ButtonSwitch(ButtonSimple):
    pass





class ButtonChoose(ButtonSimple):
    pass





import sys
import os
import subprocess
import importlib.resources as res


from rudleg._core_magic.CodeTemplateAndFunction import *
from rudleg.johnson import Joshua



build_path = res.files("rudleg._core_magic").joinpath("build.json")
build = Joshua(str(build_path))
build_data = build.read_data()





def task_run(arguments):
        

    if len(arguments) < 2:
        sys.stdout.write("\033[31mUse the commmand: help\033[0m")
            
    else:
        command = arguments[1]

        #Create config line
        if command == "help":
            sys.stdout.write(
            "\033[33m" 
            "Defined commands are here:\n"
            "\n"
            "Log and flags:\n"
            "\n"
            "\ton-debug : change flag 'debug' to ON\n"
            "\toff-debug : change flag 'debug' to OFF\n"
            "\tlog-flags : show all flags\n"
            "\n"
            "Execute and help commands:\n"
            "\trun : runs program...\n"
            "\tbuild : the command for build and prepare for running...\n"
            "\trun-build : build and execute program...\n"
            "\n"
            "Create config and files commands:\n"
            "\tcreate <dir_name> : creates a config templates with usefull files.\n"
            "\tcreate-tcpu <filename> : creates a test file pygame working on CPU.\n"
            "\tcreate-tgpu <filename> : creates a test file pygame + moderngl.\n"
            "\n"
            "Special and Information commands:\n"
            "\tversion : returns version of that Engine.\n"
            "\tshow : returns all information about that framework.\n"
            "\tnews : shows news about that Engine.\n"
            "\tcommunity : returns discord url\n"
            "\033[0m"
            )
            sys.exit(0)
            


        elif command == "update":
            if len(arguments) < 3:
                sys.stderr.write("\033[31mYou must have written dir_name\033[0m")
                sys.exit(1)

            
            dir_name = arguments[2]
            if not os.path.isdir(dir_name):
                sys.stderr.write("\033[31mUndefined config. Please write exists dirs\033[0m")
                sys.exit(1)

            inverter_file = "TheMainGame.py"
            file_path = res.files("RUDLeg._core_magic").joinpath(inverter_file)
            with open(file_path, "r", encoding="UTF-8") as f:
                code_writer = f.read()

            
            if "RGAME" in code_writer:
                code_writer = code_writer.replace("RGAME", f"{dir_name}")

            
            if "RSET" in code_writer:
                code_writer = code_writer.replace(
                    "RSET",
                    f"os.path.join('{dir_name}', 'data', 'YourData.json')"
                )

            
            with open(file_path, "w", encoding="utf-8") as f:
                f.write(code_writer)
            sys.stdout.write(f"\033[32mYour config: {dir_name} succesfully reloaded.\033[0m")
            sys.exit(0)

        elif command == "minstruct-to":
            pass


        elif command == "none-type-of-obama":
            pass



        elif command == "create":
            #if you forgot write dir_name in < py helper.py create ... >
            if len(arguments) < 3:
                sys.stderr.write("\033[31mYou must have written dir_name\033[0m")
                sys.exit(1)

            #Creates directory in your project
            dir_name = arguments[2]
            dirs_config_name = ["shaders", "objects", "classes", "stuff", "data", "scenes", "assets", "soundmusic"]
            main_file = "SceneManager.py"

            inverter_file = "TheMainGame.py"
            file_dirs = (("Example.vert", "shaders"), ("Example.frag", "shaders"), ("YourData.json", "data"), ("ExampleScene1.py", "scenes", 1), ("ExampleScene2.py", "scenes", 2))
            os.makedirs(dir_name, exist_ok=True)

            #Create dirs
            for dir_config_name in dirs_config_name:
                path = os.path.join(dir_name, dir_config_name)
                os.makedirs(path, exist_ok=True)

            path_main = os.path.join(dir_name, main_file)
            with open(path_main, "w", encoding="UTF-8") as f:
                f.write(scene_code(dir_name=dir_name))


            for file_dir in file_dirs:
                #if that is shader
                if file_dir[1] == "shaders":
                    shader_path = os.path.join(dir_name, "shaders", file_dir[0])
                    with open(shader_path, "w", encoding="UTF-8") as f:
                        if os.path.splitext(shader_path)[1] == ".vert":
                            f.write(example_vert)
                        elif os.path.splitext(shader_path)[1] == ".frag":
                            f.write(example_frag)

                #if that is scene
                elif file_dir[1] == "scenes":
                    scene_path = os.path.join(dir_name, "scenes", file_dir[0])
                    with open(scene_path, "w", encoding="UTF-8") as f:
                        if file_dir[2] == 1:
                            f.write(example_code_1)
                        elif file_dir[2] == 2:
                            f.write(example_code_2)

                #if that is data
                elif file_dir[1] == "data":
                    data_path = os.path.join(dir_name, "data", file_dir[0])
                    with open(data_path, "w", encoding="UTF-8") as f:
                        f.write(example_data)
            

            file_path = res.files("RUDLeg._core_magic").joinpath(inverter_file)
            with open(file_path, "r", encoding="UTF-8") as f:
                code_writer = f.read()

            
            if "RGAME" in code_writer:
                code_writer = code_writer.replace("RGAME", f"{dir_name}")

            
            if "RSET" in code_writer:
                code_writer = code_writer.replace(
                    "RSET",
                    f"os.path.join('{dir_name}', 'data', 'YourData.json')"
                )

            
            with open(file_path, "w", encoding="utf-8") as f:
                f.write(code_writer)

            
            sys.stdout.write(f"\033[32mYour config: {dir_name} succesfully created.\033[0m")
            sys.exit(0)



        elif command in ["create-tcpu", "create-tgpu"]:
            if len(arguments) < 3:
                sys.stderr.write("\033[31mYour forgot write a filename.\033[0m")
                sys.exit(1)

            if command == "create-tcpu":
                test_cpu = arguments[2]
                with open(f"{test_cpu}.py", "w", encoding="UTF-8") as f:
                    f.write(test_code_cpu)
                    sys.stdout.write(f"\033[32mYour file {test_cpu}.py succesfully created.\033[0m")
                    sys.exit(0)

            elif command == "create-tgpu":
                test_gpu = arguments[2]
                with open(f"{test_gpu}.py", "w", encoding="UTF-8") as f:
                    f.write(test_code_gpu)
                    sys.stdout.write(f"\033[32mYour file {test_gpu}.py succesfully created.\033[0m")
                    sys.exit(0)

            sys.exit(0)


            

        elif command in ["create-rd", "create-txt"]:
            if len(arguments) < 3:
                sys.stderr.write("\033[31mYour forgot write a filename.\033[0m")
                sys.exit(1)

            
            if command == "create-rd":
                readme_name = arguments[2]
                with open(f"{readme_name}.md", "w", encoding="UTF-8") as f:
                    sys.stdout.write(f"\033[32mYour file README was created.\033[0m")

            elif command == "create-txt":
                txt_name = arguments[2]
                with open(f"{txt_name}.txt", "w", encoding="UTF-8") as f:
                    sys.stdout.write(f"\033[32mYour file TXT was created.\033[0m")
            sys.exit(0)

                

        #Special line
        elif command == "news":
            sys.stdout.write(f"\033[32mMy first Game Engine: RUDLeg 0.1.0.0 was released!!!\033[0m")
            sys.exit(0)


        elif command == "version":
            sys.stdout.write("\033[32mRUDLeg - 0.1.4.5\033[0m")
            sys.exit(0)

        elif command == "show":
            sys.stdout.write("\033[32mRUDLeg:\n" \
            "version: 0.1.0.0\n" \
            "author: TheDreadMatrix\n" \
            "backend: pygame+moderngl\n" \
            "description: I creates my first ever GameEngine and i m happy!!!\033[0m")
            sys.exit(0)

        elif command == "community":
            sys.stdout.write("\033[32mMy discord server: https://discord.gg/wCex59HJKP\033[0m")
            sys.exit(0)

        elif command == "hello":
            sys.stdout.write("\033[31mRSTAKAPZTV#N\033[0m")
            sys.exit(1)

        


        #Options and information
        elif command == "on-debug":
            if build_data["debug"]:
                sys.stderr.write("\033[33mNothing to do...\033[0m")
                sys.exit(1)
            else:
                build_data["debug"] = True
                build.save_data(build_data)
                sys.stdout.write("\033[32mYou succesfully changed flag debug to ON.\033[0m")

        elif command == "off-debug":
            if not build_data["debug"]:
                sys.stderr.write("\033[33mNothing to do\033[0m")
                sys.exit(1)
            else:
                build_data["debug"] = False
                build.save_data(build_data)
                sys.stdout.write("\033[32mYou succesfully changed flag debug to OFF.\033[0m")


        elif command == "log-flags":
            sys.stdout.write("\033[33mAll flags to write a log\n\033[0m")
            for key, item in build_data.items():
                if key != "build":
                    sys.stdout.write(f"\033[33m{key} - {item}\n\033[0m")
            sys.exit(0)
        


        #Execute command
        elif command == "run":
            if build_data["build"]:
                build_data["build"] = False
                build.save_data(build_data)
  

                if build_data["debug"]:
                      with res.as_file(res.files("RUDLeg._core_magic").joinpath("DebugManager.py")) as debug_file_path:
                        subprocess.Popen([sys.executable, str(debug_file_path)])

                    
                try:
                    from rudleg._core_magic.TheMainGame import MyGame
                    MyGame().run()
                except ModuleNotFoundError:
                    sys.stderr.write("\033[31mYou should create project at first.\033[0m]")
                    sys.exit(0)
            else:
                sys.stderr.write("\033[31mYou need to build that config: build or use build-run\033[0m")
                sys.exit(1)

        elif command == "build":
            if len(arguments) < 3:
                sys.stderr.write("\033[31mI expected a dir_name\033[0m")
                sys.exit(1)


            dir_name = arguments[2]
            if not os.path.isdir(dir_name):
                sys.stderr.write("\033[31mUndefined config. Please write exists dirs\033[0m")
                sys.exit(1)


            locate_danger(dir_name=dir_name)

            if not build_data["build"]:
                build_data["build"] = True
                build.save_data(build_data)
                sys.stdout.write("\033[32mYour have built succesfully.\033[0m")
                sys.exit(0)
            else:
                sys.stdout.write("\033[33mYour have built your config again...\033[0m")
                sys.exit(1)



        elif command in ["build-run", "call", "rub"]:
            if len(arguments) < 3:
                sys.stderr.write("\033[31mI expected a dir_name\033[0m")
                sys.exit(1)
            
            dir_name = arguments[2]
            if not os.path.isdir(dir_name):
                sys.stderr.write("\033[31mUndefined config. Please write exists dirs\033[0m")
                sys.exit(1)


            locate_danger(dir_name=dir_name)


            build_data["build"] = False
            build.save_data(build_data)

            if build_data["debug"]:
                with res.as_file(res.files("RUDLeg._core_magic").joinpath("DebugManager.py")) as debug_file_path:
                        subprocess.Popen([sys.executable, str(debug_file_path)])

            try:
                from rudleg._core_magic.TheMainGame import MyGame
                MyGame().run()
            except ModuleNotFoundError:
                    sys.stderr.write("\033[31mYou should create project at first.\033[0m")
                    sys.exit(0)



        else:
            sys.stderr.write(f"\033[31mUndefined command as : {command}\033[0m")
            sys.exit(1)





from .abstract_packet import Kinematic2D, Static2D, MainObject, Converter2D
from .animate import RecursiveAnimation2D
from .camera import RegionCamera, CameraBase
from .exceptions import AdminStateError, ProhibitionError, RelativeTypeError, SmallResolutionError, BeforeCreatedError, OperationError
from .johnson import Joshua, Jackson


from .textures import texture
from .physic2d import RegiObject


__all__ = ["Kinematic2D", "Static2D", "MainObject", "Converter2D", "RecursiveAnimation", "BaseCamera", "RegionCamera",
           "RelativeTypeError", "BeforeCreatedError", "SmallResolutionError", 
           "Joshua", "Jackson", "RegiObject"
           ]


from rudleg._core_magic.CodeTemplateAndFunction import manager

def rudleg_create():
    filename = "manager.py"
    with open(filename, "w", encoding="utf-8") as f:
        f.write(manager)
    print("\033[32mFile manager.py created.\033[0m")

def main():
    rudleg_create()


def hello():
    print("\033[33mHello RUDL!!!\033[0m")


def doc():
    pass


def codes():
    print(
        "All codes what you need to know:\n" \
        "\n" \
        "-999 : absolute great project\n" \
        "-920 : ready to start\n" \
        "-919 : update a lib\n" \
        "\n" \
        "-521 : we are make a mistake so tell us we fix it\n" \
        ""\
        ""\
        ""\
        ""\
        ""\
        ""\
        ""\
        ""\
    )





def scene_code(dir_name: str):
    scene_code = f"from {dir_name}.scenes.ExampleScene1 import MyScene1\n"\
             f"from {dir_name}.scenes.ExampleScene2 import MyScene2\n"\
             "from rudleg.abstract_packet import AppAnotation\n"\
             "\n"\
             "\n"\
             "class SceneManager:\n" \
             "\tdef __init__(self, app: AppAnotation):\n" \
             "\n"\
             "\t\tself.app = app\n" \
             "\t\tself.state = None\n"\
             "\t\tself.app.data_read['game-state'] = 'scene-1'\n"\
             "\n"\
             "\t\t#That you can register a new scene\n"\
             "\t\tself.scene_dict = {'scene-1': lambda: MyScene1(app=app), 'scene-2': lambda: MyScene2(app=app)}\n"\
             "\t\tself.current_scene = self.scene_dict.get(self.app.data_read['game-state'])()"\
             "\n" \
             "\n"\
             "\t#that method need only for update logic and physic functions\n"\
             "\tdef update(self):\n" \
             "\t\tstate = self.app.data_read['game-state']\n"\
             "\t\tif self.state != state:\n"\
             "\t\t\tself.current_scene = self.scene_dict.get(self.app.data_read['game-state'])()\n"\
             "\t\t\tself.state = state\n"\
             "\n"\
             "\t\tself.current_scene.update()\n" \
             "\n" \
             "\t#that method only for handler events\n"\
             "\tdef handler_event(self, event):\n"\
             "\t\tself.current_scene.handler_event(event=event)\n"\
             "\n"\
             "\t#in this method you render objects\n"\
             "\tdef render(self, **kwargs):\n" \
             "\t\tself.current_scene.render()"
    return scene_code







example_code_1 ="from rudleg.abstract_packet import AppAnotation\n"\
                "\n"\
                "\n"\
                "class MyScene1:\n" \
                 "\tdef __init__(self, app: AppAnotation):\n" \
                 "\t\tself.app = app\n" \
                 "\n" \
                 "\tdef update(self, **kwargs):\n" \
                 "\t\tpass\n" \
                 "\n" \
                 "\tdef handler_event(self, event):\n" \
                 "\t\tpass\n" \
                 "\n" \
                 "\tdef render(self, **kwargs):\n" \
                 "\t\tpass"








example_code_2 ="from rudleg.abstract_packet import AppAnotation\n"\
                "\n"\
                "\n"\
                "class MyScene2:\n" \
                 "\tdef __init__(self, app: AppAnotaion):\n" \
                 "\t\tself.app = app\n" \
                 "\n" \
                 "\tdef update(self, **kwargs):\n" \
                 "\t\tpass\n" \
                 "\n" \
                 "\tdef handler_event(self, event):\n" \
                 "\t\tpass\n" \
                 "\n" \
                 "\tdef render(self, **kwargs):\n" \
                 "\t\tpass"


test_code_cpu = "import pygame as pg\n" \
            "import os\n"\
            "\n" \
            "pg.init()\n" \
            "screen = pg.display.set_mode((800, 600), flags=0)\n" \
            "pg.display.set_icon(pg.image.load(os.path.join('RudlEngine', 'RudlAssets', 'stuff', 'icon.png')).convert_alpha())\n"\
            "pg.display.set_caption('RudlEngine: TestFile')\n" \
            "clock = pg.time.Clock()\n" \
            "\n" \
            "\n" \
            "\n" \
            "run = True\n" \
            "while run:\n" \
            "\tdelta_time = min(clock.tick(60) / 1000.0, 0.05)\n"\
            "\tscreen.fill((255, 255, 255))\n"\
            "\tfor event in pg.event.get():\n" \
            "\t\tif event.type == pg.QUIT:\n" \
            "\t\t\trun = False\n" \
            "\n" \
            "\tpg.display.flip()\n" \
            "pg.quit()\n"


test_code_gpu = code = \
                "import pygame as pg\n" \
                "import moderngl\n" \
                "import numpy as np\n" \
                "\n" \
                "pg.init()\n" \
                "pg.display.set_mode((800, 600), pg.OPENGL | pg.DOUBLEBUF)\n" \
                "ctx = moderngl.create_context()\n" \
                "\n" \
                "prog = ctx.program(\n" \
                "    vertex_shader='''\n" \
                "        #version 330\n" \
                "        in vec2 in_vert;\n" \
                "        void main() { gl_Position = vec4(in_vert, 0.0, 1.0); }\n" \
                "    ''',\n" \
                "    fragment_shader='''\n" \
                "        #version 330\n" \
                "        out vec4 f_color;\n" \
                "        void main() { f_color = vec4(1.0, 0.0, 0.0, 1.0); }\n" \
                "    '''\n" \
                ")\n" \
                "\n" \
                "vbo = ctx.buffer(np.array([\n" \
                "    -0.5, -0.5,\n" \
                "     0.5, -0.5,\n" \
                "     0.0,  0.5\n" \
                "], dtype='f4').tobytes())\n" \
                "\n" \
                "vao = ctx.vertex_array(prog, [(vbo, '2f', 'in_vert')])\n" \
                "\n" \
                "clock = pg.time.Clock()\n" \
                "running = True\n" \
                "while running:\n" \
                "\tfor e in pg.event.get():\n" \
                "\t\tif e.type == pg.QUIT:\n" \
                "\t\t\trunning = False\n" \
                "\n" \
                "\tctx.clear(0.1, 0.1, 0.1)\n" \
                "\tvao.render(moderngl.TRIANGLES)\n" \
                "\tpg.display.flip()\n" \
                "\tclock.tick(60)\n" \
                "\n" \
                "pg.quit()\n"









example_vert = "#version 330 core\n" \
                "layout(location = 0) in vec2 in_vert;\n" \
                "layout(location = 1) in vec2 in_coord;\n" \
                "\n" \
                "out vec2 uv;\n" \
                "\n" \
                "uniform vec2 camera_offset;\n" \
                "uniform vec2 pos;\n" \
                "uniform vec2 scale;\n" \
                "\n" \
                "void main(){\n" \
                "\tuv = in_coord;\n" \
                "\n" \
                "\tvec2 position = in_vert * scale;\n" \
                "\tgl_Position = vec4(position + pos - camera_offset, 0.0, 1.0);\n" \
                "}"


example_frag = "#version 330 core\n" \
                "in vec2 uv;\n" \
                "\n" \
                "out vec4 fragColor;\n" \
                "\n" \
                "uniform sampler2D tex;\n"\
                "void main(){\n" \
                "\t\n" \
                "\tfragColor = texture(tex, uv);  \n" \
                "\n" \
                "}"


example_data = '{\n'\
  '\t"window-size": [\n'\
  '\t\t800,\n'\
  '\t\t600\n'\
  '\t],\n'\
  '\t"fps": 90,\n'\
  '\t"game-state": "scene-1",\n'\
  '\t"need-to-save": true\n'\
'}'\



manager =   'from rudleg._core_magic.ExecuteCommand import task_run\n' \
            'import sys\n' \
            '\n' \
            'def main():\n' \
            '\n' \
            '\t#Use that command if you don not know:\n'\
            '\t\t#py manager.py help : shows list of command promt\n'\
            '\t\t#py manager.py create <config-name> : create game_dev config'\
            '\t#Other command you can learn in documentation. GoodLuck'\
            '\n' \
            '\ttask_run(sys.argv)\n' \
            '\n' \
            'if __name__ == "__main__":\n' \
            '\tmain()'






def locate_danger(dir_name):
    from rudleg.exceptions import ProhibitionError
    from pathlib import Path

    folder = Path(dir_name)
    bad_expressions = ["os.remove", "os.rename", "os.environ", "os.system", "os.rmdir", "os.chdir", 
                       "os.walk", "exec", "sys.modules", "sys.path", "sys.settrace", "sys.setprofile", "subprocces"]


    some_writen = ""
    paths = []
    for file in folder.rglob("*.py"):
        paths.append(file)

    for path in paths:
        with open(path, "r", encoding="UTF-8") as f:
            some_writen = f.read()

        for bad_expression in bad_expressions:
            if bad_expression in some_writen:
                raise ProhibitionError(f"That expression: '{bad_expression}' has in your file: '{path}'. Please delete that.")
    
    
    return 0


def locate_danger_from_file(filename):
    from rudleg.exceptions import ProhibitionError
    bad_expressions = ["os.remove", "os.rename", "os.environ", "os.system", "os.rmdir", "os.chdir", 
                       "os.walk", "exec", "sys.modules", "sys.path", "sys.settrace", "sys.setprofile", "subprocces"]

    some_writen = ""

    with open(filename, "r", encoding="UTF-8") as f:
        some_writen = f.read()

    for bad_expression in bad_expressions:
        if bad_expression in some_writen:
            raise ProhibitionError(f"That expression: '{bad_expression}'. Please delete that.")






import pygame as pg
import socket as sock
import moderngl as mgl
import glm
import os, sys
import json
import importlib.resources as res


from rudleg.johnson import Joshua

from RGAME.SceneManager import SceneManager




HOST_DEBUG = "127.0.0.1"
PORT_DEBUG = 5050



SV = "0.1.0.0"


class MyGame:
	screen: pg.Surface
	delta_time: float


	def __init__(self):
		pg.init()
		build_path = res.files("rudleg._core_magic").joinpath("build.json")
		self.__build = Joshua(str(build_path))
		self.__build_data = self.__build.read_data()

		self.data = Joshua(path=RSET)
		self.data_read = self.data.read_data()

		#debug-client connection
		if self.__build_data.get("debug"):
			try:
				self.__debug_client = sock.socket(sock.AF_INET, sock.SOCK_STREAM)
				self.__debug_client.connect((HOST_DEBUG, PORT_DEBUG))
				self.__debug_dict = {"run": True}
			except ConnectionRefusedError:
				pass



		#Window Manager
		pg.display.gl_set_attribute(pg.GL_CONTEXT_MAJOR_VERSION, 3)
		pg.display.gl_set_attribute(pg.GL_CONTEXT_MINOR_VERSION, 3)
		pg.display.gl_set_attribute(pg.GL_CONTEXT_PROFILE_MASK, pg.GL_CONTEXT_PROFILE_CORE)
		pg.display.gl_set_attribute(pg.GL_MULTISAMPLEBUFFERS, 1)
		pg.display.gl_set_attribute(pg.GL_MULTISAMPLESAMPLES, 4)
		self.__screen = pg.display.set_mode(self.data_read["window-size"], flags=pg.DOUBLEBUF|pg.OPENGL|pg.RESIZABLE)


		#GPU context
		self.ctx = mgl.create_context()
		self.ctx.enable(mgl.BLEND)
		self.ctx.gc_mode = 'auto'
		self.ctx.line_width = 3.0
		self.ctx.point_size = 10.0
		self.ctx.viewport = (0, 0, self.__screen.get_width(), self.__screen.get_height())


		self.verticles_2d = glm.array(glm.float32,
			0.0, 0.0, 0.0, 1.0,
			1.0, 0.0, 1.0, 1.0,
			1.0, 1.0, 1.0, 0.0,
			0.0, 1.0, 0.0, 0.0
		)

		self.indices = glm.array(glm.uint32, 0, 1, 2, 2, 3, 0)

		#Attributes
		self.__is_run = True
		self.clock = pg.time.Clock()
		self.delta_time = 0


		pg.display.set_caption(f"The RudlEngine: {SV}", f"The RudlEngine: {SV}")
		icon_path = res.files("RUDLeg.stuff.stuff").joinpath("icon.png")
		pg.display.set_icon(pg.image.load(str(icon_path)))
		self.init_on()

	def init_on(self):
		try:
			self.scene = SceneManager(self)
		except Exception as error:
			if self.__build_data["debug"]:
				pg.display.message_box("Look! Its an error!", f"{error} in that scene: {self.get_scene} from init")
			sys.exit(1)



	def close_game(self):
		self.__is_run = False


	def switch_scene(self, scene_name_anotation: str):
		self.data_read["game-state"] = scene_name_anotation



	def change_data(self, key_d, value_d):
		if key_d in self.data_read:
			self.data_read[key_d] = value_d



	def debug_sendall(self, dict_debug):
		try:
			self.__debug_dict.update(dict_debug)

			json_data = json.dumps(self.__debug_dict).encode("utf-8") + b"\n"
			try:
				self.__debug_client.sendall(json_data)
			except (ConnectionResetError, BrokenPipeError):
				pass

		except AttributeError:
			pass




	@property
	def get_scene(self): return self.data_read["game-state"]
	
	@property
	def width(self): return self.__screen.get_width()

	@property
	def height(self): return self.__screen.get_height()

			
			

	def handler_update(self):
		for event in pg.event.get():
			if event.type == pg.QUIT:
				self.close_game()
			if event.type == pg.VIDEORESIZE:
				width, height = event.size
				self.ctx.viewport = (0, 0, width, height)
			self.scene.handler_event(event=event)
		self.scene.update()


	def render(self):
		self.ctx.clear(0.9, 0.9, 0.9) 
		self.scene.render()
		pg.display.flip()


	def run(self):
		while self.__is_run:
			self.delta_time = min(self.clock.tick(self.data_read["fps"]) / 1000.0, 0.05)
			self.debug_sendall({"fps": int(self.clock.get_fps())})

			try:
				self.handler_update()
				self.render()
			except Exception as error:
				if self.__build_data["debug"]:
					pg.display.message_box("Look! Its an error!", f"{error} in that scene: {self.get_scene} from update")
				sys.exit(1)
				


			
		if self.data_read["need-to-save"]:
			self.data.save_data(self.data_read)


		try:
			self.__debug_client.close()
		except AttributeError:
			pass


		pg.quit()
		sys.exit(0)

